#include "window.h"

#include <QtGui/QMenuBar>

#include "view.h"

namespace Kritzel
{

class Window::Private
{
public:
    Window *q;
    View *view;
    QWidget *central;
};

Window::Window(QWidget *parent)
    : QMainWindow(parent)
    , d(new Private)
{
    d->q = this;

    QAction *quitAction = menuBar()->addAction("Quit");
    connect(quitAction, SIGNAL(triggered()), this, SLOT(close()));

    QAction *clearAction = menuBar()->addAction("Clear");
    connect(clearAction, SIGNAL(triggered()), this, SLOT(clear()));

    QAction *zoomInAction = menuBar()->addAction("Zoom +");
    connect(zoomInAction, SIGNAL(triggered()), this, SLOT(zoomIn()));

    QAction *zoomOutAction = menuBar()->addAction("Zoom -");
    connect(zoomOutAction, SIGNAL(triggered()), this, SLOT(zoomOut()));

    d->view = new View(this);
    d->view->resize(0, 0);
    setCentralWidget(d->view);
}

Window::~Window()
{
    delete d;
}

void Window::clear()
{
    d->view->scene()->clear();
}

void Window::zoomIn()
{
    d->view->scale(2, 2);
}

void Window::zoomOut()
{
    d->view->scale(0.5, 0.5);
}

QSize Window::sizeHint() const
{
    int fontHeight = fontMetrics().height();
    return QSize(30 * fontHeight, 40 * fontHeight);
}

} // namespace Kritzel

